import pygamejr
import bullets
import enemy_bullets

fortifications = []
for bottom in range(70, 110, 15):
   for left in range(100, pygamejr.screen.get_width() - 100, 150):
       fortification = pygamejr.RectSprite(width=50, height=10)
       fortification.rect.bottom = pygamejr.screen.get_height() - bottom
       fortification.rect.left = left
       fortifications.append(fortification)

def is_hit_by_bullet():
   '''проверяет попала ли наша пуля в укрепление'''
   for fortification in fortifications:
       if bullets.is_hit_enemy(fortification):
           fortification.is_visible = False
           fortifications.remove(fortification)
           return True
   return False

def is_hit_by_enemy_bullet():
   '''проверяет попала ли вражеская пуля в укрепление'''
   for fortification in fortifications:
       if enemy_bullets.is_hit_enemy(fortification):
           fortification.is_visible = False
           fortifications.remove(fortification)
           return True
   return False



from codomir import player, wait_quit, set_map, maps

set_map(maps.nested_loops.map1)

player.turn_right()
player.turn_right()

for i in range(1, 6):
    for j in range(i):
        player.move_forward()
    player.turn_right()

wait_quit()


import os
import pygame

pygame.init()

window_width = int(os.environ.get('PYGAMEJR_WINDOW_WIDTH') or 800)
window_height = int(os.environ.get('PYGAMEJR_WINDOW_HEIGHT') or 600)

screen = pygame.display.set_mode((window_width, window_height), pygame.RESIZABLE)

clock = pygame.time.Clock()

_global_scene = None    # always rendered on top
_current_scene = None   # set by set_scene(), rendered below global


def get_global_scene():
    """Returns the global scene. Sprites added here are always visible regardless of current scene."""
    global _global_scene
    if _global_scene is None:
        from .scene import Scene
        _global_scene = Scene()
    return _global_scene


def get_current_scene():
    """Returns the current scene set by set_scene(), or None if not set."""
    return _current_scene


def set_scene(scene) -> None:
    global _current_scene
    _current_scene = scene
    if scene is not None:
        scene.init_scene()


def next_frame():
    dt = clock.tick(60) / 1000
    screen.fill("black")
    if _current_scene is not None:
        _current_scene.update(dt)
        _current_scene.draw()
    global_scene = get_global_scene()
    global_scene.update(dt)
    global_scene.draw()
    pygame.display.flip()
    return not is_quit()


def every_frame(frame_count=0, draw_sprites_rect=False):
    running = True
    frame = -1
    while running:
        dt = clock.tick(60) / 1000

        if is_quit() or frame >= frame_count:
            break

        if frame_count:
            frame += 1

        screen.fill("black")

        yield dt

        if _current_scene is not None:
            _current_scene.update(dt)
            _current_scene.draw(draw_rect=draw_sprites_rect)

        global_scene = get_global_scene()
        global_scene.update(dt)
        global_scene.draw(draw_rect=draw_sprites_rect)

        pygame.display.flip()


_is_quit = False


def is_quit():
    global _is_quit

    if _is_quit:
        return True

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            _is_quit = True
            return True
    return False


def wait_quit():
    for _ in every_frame():
        pass



import pygamejr
import pygame

tilemap = pygamejr.TileMap('map/map.tmx')
background = tilemap.get_layer_sprites('background')
obstacles = tilemap.get_layer_sprites('obstacles')

player = tilemap.get_sprite_by_tileset_position(3, 12)
player.rect.centerx = 32
player.rect.centery = 4.5 * 64

foreground = tilemap.get_layer_sprites('foreground')

loot = tilemap.get_layer_sprites('loot')

for frame in pygamejr.every_frame(draw_sprites_rect=True):

    keys = pygame.key.get_pressed()

    rect = player.rect.copy()
    if keys[pygame.K_w]:
        player.rect.centery -= 1
    if keys[pygame.K_s]:
        player.rect.centery += 1
    if keys[pygame.K_d]:
        player.rect.centerx += 1
    if keys[pygame.K_a]:
        player.rect.centerx -= 1

    for sprite in obstacles:
        if player.rect.colliderect(sprite.rect):
            player.rect = rect
            break

    thing_for_delete = None
    for thing in loot:
        if player.rect.colliderect(thing.rect):
            thing_for_delete = thing

    if thing_for_delete:
        thing_for_delete.is_visible = False
        loot.remove(thing_for_delete)



from pygame.sprite import Sprite as PygameSprite, AbstractGroup, LayeredUpdates
from .tilemap import TileMap

DEFAULT_LAYER = "default"

class Scene:
    def __init__(self):
        self._name_mapping: dict[str, AbstractGroup] = {
            DEFAULT_LAYER: LayeredUpdates()
        }
        self.pending_transition: str | None = None

    @classmethod
    def from_tilemap(cls, tilemap: TileMap) -> "Scene":
        from .base import get_current_scene, set_scene

        scene = cls()
        previous_scene = get_current_scene()  # may be None
        set_scene(scene)

        for layer in tilemap.tmxdata.layers:
            if hasattr(layer, 'tiles'):
                sprites = tilemap.get_layer_sprites(layer.name)
                scene.add_sprite_list(name=layer.name, sprite_list=LayeredUpdates(sprites))

        scene._name_mapping[DEFAULT_LAYER].empty()
        set_scene(previous_scene)
        return scene

    def add(self, sprite: PygameSprite) -> None:
        self._name_mapping[DEFAULT_LAYER].add(sprite)

    def remove(self, sprite: PygameSprite) -> None:
        self._name_mapping[DEFAULT_LAYER].remove(sprite)

    def add_sprite_list(self, name: str, sprite_list: AbstractGroup) -> None:
        self._name_mapping[name] = sprite_list

    def get_sprite_list(self, name: str) -> AbstractGroup | None:
        return self._name_mapping.get(name)

    def init_scene(self) -> None:
        """Called when this scene becomes the active scene. Override to add initialization logic."""
        pass

    def update(self, dt: float = 0) -> None:
        for sprite_list in self._name_mapping.values():
            sprite_list.update(dt)

    def draw(self, draw_rect: bool = False) -> None:
        for sprite_list in self._name_mapping.values():
            for sprite in sprite_list:
                sprite.draw(draw_rect=draw_rect)



from .base import every_frame, wait_quit, is_quit, screen, next_frame, set_scene, get_current_scene, get_global_scene
from .sprite.circle import CircleSprite
from .sprite.image import ImageSprite
from .sprite.rect import RectSprite
from .sprite.text import TextSprite
from .sprite.subtitles import SubtitlesSprite
from .tilemap import TileMap
from .scene import Scene
from . import resources

__all__ = [
    every_frame,
    wait_quit,
    screen,
    is_quit,
    next_frame,
    set_scene,
    get_current_scene,
    get_global_scene,
    CircleSprite,
    ImageSprite,
    RectSprite,
    TileMap,
    Scene,
    resources,
]



from ..resources.resolve_path import resolve_path

map1 =  resolve_path('kenney_sokoban/map1.tmx')
map2 = resolve_path('kenney_sokoban/map2.tmx')
map3 = resolve_path('kenney_sokoban/map3.tmx')
map4 = resolve_path('kenney_sokoban/map4.tmx')
map5 = resolve_path('kenney_sokoban/map5.tmx')
map6 = resolve_path('kenney_sokoban/map6.tmx')


from typing import Union
from pathlib import Path

#: The absolute path to this directory
RESOURCE_PATH = Path(__file__).parent.absolute()


def resolve_path(path: Union[str, Path]) -> Path:
    path = Path(RESOURCE_PATH / path)
    return path.absolute()



from codomir import player, wait_quit, set_map, maps

set_map(maps.loop.map1)

for i in range(3):
    player.move_forward()

player.turn_left()

for i in range(2):
    player.move_forward()

player.turn_right()

for i in range(2):
    player.move_forward()

player.turn_right()

for i in range(5):
    player.move_forward()

player.turn_right()

for i in range(2):
    player.move_forward()

player.turn_right()
player.move_forward()
player.turn_left()

for i in range(3):
    player.move_forward()

player.turn_left()

player.move_forward()

wait_quit()



import pygamejr

width = pygamejr.screen.get_width()
height = pygamejr.screen.get_height()

top_left = pygamejr.ImageSprite(pygamejr.resources.image.coin_gold)
top_left.rect.top = 0
top_left.rect.left = 0

top_right = pygamejr.ImageSprite(pygamejr.resources.image.coin_bronze)
top_right.rect.top = 0
top_right.rect.right = width

bottom_left = pygamejr.ImageSprite(pygamejr.resources.image.coin_gold)
bottom_left.rect.bottom = height
bottom_left.rect.left = 0

bottom_right = pygamejr.ImageSprite(pygamejr.resources.image.coin_gold)
bottom_right.rect.bottom = height
bottom_right.rect.right = width

center = pygamejr.ImageSprite(pygamejr.resources.image.coin_silver)
center.rect.centerx = pygamejr.screen.get_width() / 2
center.rect.centery = pygamejr.screen.get_height() / 2

pygamejr.wait_quit()



import pygame
import pygamejr

bee = pygamejr.ImageSprite(pygamejr.resources.image.bee)
slime = pygamejr.ImageSprite(pygamejr.resources.image.slime_block)
slime.rect.centerx = pygamejr.screen.get_width()/2
slime.rect.centery = pygamejr.screen.get_height()/2

SPEED= 5

for frame in pygamejr.every_frame():
    # Получаем состояние всех клавиш
    keys = pygame.key.get_pressed()

    # Создадим копию прямоугольника описывающего спрайт
    rect = bee.rect.copy()
    # Двигаем прямоугольник в зависимости от нажатых клавиш
    if keys[pygame.K_w]:
        rect.centery -= SPEED
    if keys[pygame.K_s]:
        rect.centery += SPEED
    if keys[pygame.K_a]:
        rect.centerx -= SPEED
    if keys[pygame.K_d]:
        rect.centerx += SPEED

    # Обновим положение пчелки только если пчелка не касается слфайм
    if not slime.rect.colliderect(rect):
        bee.rect.update(rect)



import os
from pathlib import Path

from pygame.sprite import Sprite as PygameSprite
from pygame import Rect
from pytmx.util_pygame import load_pygame, pygame_image_loader

from .sprite.image import ImageSprite


# def get_tile_by_local_id(tmx_data, local_id, tileset_name=None):
#     """
#     tileset_name: Имя набора тайлов (как в Tiled)
#     local_id: Номер картинки внутри набора (0, 1, 2...)
#     """
#     # Ищем нужный тайлсет
#     target_tileset = None
#     # for ts in tmx_data.tilesets:
#     #     if ts.name == tileset_name:
#     #         target_tileset = ts
#     #         break
#     target_tileset = tmx_data.tilesets[0]
#
#     if target_tileset:
#         # Вычисляем настоящий GID
#         real_gid = target_tileset.firstgid + local_id
#
#         # Теперь это сработает, так как load_all_tiles загрузил картинку
#         return tmx_data.get_tile_image_by_gid(real_gid)
#
#     return None

# Судя по всему в pytmx есть ошибка, опция load_all_tiles не работает.
# Воркэраунд под эту ошибку.
# tmxdata.gidmap = {}
# tmxdata.reload_images()
def load_image(tilemap, row, col):
    tileset = tilemap.tilesets[0]
    colorkey = getattr(tileset, "trans", None)
    path = os.path.join(os.path.dirname(tilemap.filename), tileset.source)
    loader = pygame_image_loader(path, colorkey, tileset=tileset)

    # tileset = None
    # for ts in tmx_data.tilesets:
    #     if ts.name == tileset_name:
    #         tileset = ts
    #         break
    # tileset = tilemap.tilesets[0]
    # source_image = getattr(tileset, 'image', None)
    # if not source_image:
    #     print(f"Изображение для не загружено.")
    #     return None

    # 3. Математика координат (учитываем отступы margin и spacing)
    tw = tileset.tilewidth
    th = tileset.tileheight
    margin = tileset.margin
    spacing = tileset.spacing

    # Количество колонок в тайлсете
    # image_width - margin * 2 + spacing... упрощенно:
    width_pixel = tileset.width
    columns = (width_pixel - margin) // (tw + spacing)

    # Вычисляем координаты пикселей
    x = margin + col * (tw + spacing)
    y = margin + row * (th + spacing)

    rect = Rect(x, y, tw, th)
    return loader(rect, None)

class TileMap:
    def __init__(
            self,
            map_file: str | Path,
    ) -> None:
        self.tmxdata = load_pygame(map_file, image_loader=pygame_image_loader, load_all_tiles=True)

    def get_layer_sprites(self, layer_name: str) -> list[PygameSprite]:
        sprites = []
        layer = self.tmxdata.get_layer_by_name(layer_name)

        for x, y, image in layer.tiles():
            sprite = ImageSprite(image=image, crop_alpha=False)
            sprite.rect.x = x * self.tmxdata.tilewidth
            sprite.rect.y = y * self.tmxdata.tileheight
            sprites.append(sprite)

        return sprites

    def get_sprite_by_tileset_position(self, row, col) -> ImageSprite:
        # image = self.tmxdata.get_tile_image_by_gid(gid)
        image = load_image(self.tmxdata, row, col)
        return ImageSprite(image=image)



import time
import pygamejr
import pygame

# корабль
ship = pygamejr.ImageSprite("../assets/Ship.png")
ship.rect.bottom = pygamejr.screen.get_height() - 50

# враг
enemy = pygamejr.ImageSprite("../assets/InvaderA_00.png")

###Логика пуль#########



for frame in pygamejr.every_frame():
    keys = pygame.key.get_pressed()

    if keys[pygame.K_SPACE]:
        fire(ship)

    if enemy.is_visible and is_hit_enemy(enemy):
        enemy.is_visible = False

    bullet_fly()


from enum import Enum
import os
import json
import pygame
from pygame.sprite import Group

# https://stackoverflow.com/questions/4135928/pygame-display-position
pygame.init()

display_info = pygame.display.Info()
screen_width = display_info.current_w

window_width = 8*64
window_height = 8*64
os.environ['SDL_VIDEO_WINDOW_POS'] = "%d,%d" % (screen_width - window_width, 50)
os.environ['PYGAMEJR_WINDOW_WIDTH'] = f"{window_width}"
os.environ['PYGAMEJR_WINDOW_HEIGHT'] = f"{window_height}"

import pygamejr
from pygamejr.scene import DEFAULT_LAYER
from pygamejr.sprite.base import BaseSprite
from .maps.linear import map1

from pytmx.util_pygame import load_pygame, pygame_image_loader

tilemap = pygamejr.TileMap(map1)

CHARACTER_TILES = {
    'right':  json.loads(tilemap.tmxdata.properties['character_right'])  if 'character_right'  in tilemap.tmxdata.properties else None,
    'bottom': json.loads(tilemap.tmxdata.properties['character_bottom']) if 'character_bottom' in tilemap.tmxdata.properties else None,
    'left':   json.loads(tilemap.tmxdata.properties['character_left'])   if 'character_left'   in tilemap.tmxdata.properties else None,
    'top':    json.loads(tilemap.tmxdata.properties['character_top'])    if 'character_top'    in tilemap.tmxdata.properties else None,
}

# Workaround for a pytmx bug: load_all_tiles option does not work.
def load_image(tile_x, tile_y):
    ts = tilemap.tmxdata.tilesets[0]
    colorkey = getattr(ts, "trans", None)
    path = os.path.join(os.path.dirname(tilemap.tmxdata.filename), ts.source)
    loader = pygame_image_loader(path, colorkey, tileset=ts)
    rect = (tile_x * ts.tilewidth, tile_y * ts.tileheight, ts.tilewidth, ts.tileheight)
    return loader(rect, None)


TILE_SIZE = 64


class Direction(Enum):
    TOP = 1
    RIGHT = 2
    BOTTOM = 3
    LEFT = 4


directions = (Direction.TOP, Direction.RIGHT, Direction.BOTTOM, Direction.LEFT)


def _get_spawn_position():
    return _get_position_by_type('Spawn')


def _get_win_position():
    return _get_position_by_type('Win')


def _get_position_by_type(tile_type):
    for tile in tilemap.tmxdata.layernames['objects']:
        tile_id = tile[2]
        if tile_id and tilemap.tmxdata.tile_properties[tile_id]['type'] == tile_type:
            return tile[0], tile[1]
    return 0, 0


# =================================================================
# QUEST SCENE
#
# Loads tilemap layers and renders them in the correct order:
# map tiles first, then the player sprite on top.
# =================================================================

class QuestScene(pygamejr.Scene):
    def __init__(self, tilemap):
        super().__init__()
        # Temporarily set this scene as current so that ImageSprite
        # objects created inside get_layer_sprites() auto-add here.
        previous_scene = pygamejr.get_current_scene()
        pygamejr.set_scene(self)
        for layer in tilemap.tmxdata.layers:
            if hasattr(layer, 'tiles'):
                sprites = tilemap.get_layer_sprites(layer.name)
                self.add_sprite_list(layer.name, Group(sprites))
        # Clear DEFAULT_LAYER — tile sprites now live in named layer groups.
        self._name_mapping[DEFAULT_LAYER].empty()
        pygamejr.set_scene(previous_scene)

    def draw(self, draw_rect=False):
        # Draw tilemap layers first, then player (DEFAULT_LAYER) on top.
        for name, sprite_list in self._name_mapping.items():
            if name == DEFAULT_LAYER:
                continue
            for sprite in sprite_list:
                sprite.draw(draw_rect=draw_rect)
        for sprite in self._name_mapping[DEFAULT_LAYER]:
            sprite.draw(draw_rect=draw_rect)


# =================================================================
# Build initial scene and set it before creating Player,
# so Player auto-adds to quest_scene's DEFAULT_LAYER.
# =================================================================

quest_scene = QuestScene(tilemap)
pygamejr.set_scene(quest_scene)

win_position = _get_win_position()


class Player(BaseSprite):
    def __init__(self, *args):
        super().__init__(*args)
        self._direction = Direction.RIGHT
        self._update_image()
        self.rect = self.image.get_rect()
        tile_x, tile_y = _get_spawn_position()
        self.set_position_by_tile(tile_x, tile_y)
        self._is_end = False
        pygamejr.next_frame()

    def set_position_by_tile(self, x, y):
        self._tile_x = x
        self._tile_y = y
        self.rect.centerx = self._tile_x * TILE_SIZE + TILE_SIZE / 2
        self.rect.centery = self._tile_y * TILE_SIZE + TILE_SIZE / 2

    def _update_image(self):
        if self._direction == Direction.RIGHT and CHARACTER_TILES['right']:
            tiles = CHARACTER_TILES['right']
        elif self._direction == Direction.LEFT and CHARACTER_TILES['left']:
            tiles = CHARACTER_TILES['left']
        elif self._direction == Direction.TOP and CHARACTER_TILES['top']:
            tiles = CHARACTER_TILES['top']
        else:
            tiles = CHARACTER_TILES['bottom']
        self.images = [load_image(*tile) for tile in tiles]
        self.image = self.images[0]

    def turn_right(self):
        direction_index = directions.index(self._direction)
        self._direction = directions[(direction_index + 1) % len(directions)]
        self._update_image()
        pygamejr.next_frame()

    def turn_left(self):
        direction_index = directions.index(self._direction)
        self._direction = directions[(direction_index - 1) % len(directions)]
        self._update_image()
        pygamejr.next_frame()

    def _move_top(self):
        self._direction = Direction.TOP
        self._move_to(self._tile_x, self._tile_y - 1)

    def _move_right(self):
        self._direction = Direction.RIGHT
        self._move_to(self._tile_x + 1, self._tile_y)

    def _move_bottom(self):
        self._direction = Direction.BOTTOM
        self._move_to(self._tile_x, self._tile_y + 1)

    def _move_left(self):
        self._direction = Direction.LEFT
        self._move_to(self._tile_x - 1, self._tile_y)

    def move_forward(self):
        if self._direction == Direction.TOP:
            self._move_top()
        elif self._direction == Direction.RIGHT:
            self._move_right()
        elif self._direction == Direction.BOTTOM:
            self._move_bottom()
        else:
            self._move_left()

    def _move_to(self, tile_x: int, tile_y: int):
        if self._is_end:
            return

        dx = tile_x - self._tile_x
        dy = tile_y - self._tile_y

        if self._is_wall(tile_x, tile_y):
            self._animate_game_over(dx, dy)
            self._is_end = True
            return

        character_tiles_count = len(self.images)
        # Scene draws each frame automatically after the yield in every_frame().
        for i, dt in enumerate(pygamejr.every_frame(TILE_SIZE)):
            self.rect.x += dx
            self.rect.y += dy
            self.image = self.images[int((i // (TILE_SIZE / 6)) % character_tiles_count)]

        self.image = self.images[0]

        if self._is_win(tile_x, tile_y):
            self._animate_win()
            self._is_end = True

        self._tile_x = tile_x
        self._tile_y = tile_y

    def _is_wall(self, tile_x: int, tile_y: int):
        return tilemap.tmxdata.layernames['walls'].data[tile_y][tile_x] != 0

    def _is_win(self, tile_x: int, tile_y: int):
        return tile_x == win_position[0] and tile_y == win_position[1]

    def _animate_game_over(self, dx: float, dy: float):
        old_x = self.rect.x
        old_y = self.rect.y
        max_delta = TILE_SIZE / 5
        delta = 0
        # Scene draws each frame automatically.
        for dt in pygamejr.every_frame():
            if delta > max_delta:
                self.rect.x = old_x
                self.rect.y = old_y
                delta = 0
            else:
                self.rect.x += dx
                self.rect.y += dy
                delta += 1

    def _scale_surface(self, surf, scale: float):
        width = round(surf.get_width() * scale)
        height = round(surf.get_height() * scale)
        return pygame.transform.smoothscale(surf, (width, height))

    def _animate_win(self):
        # Temporarily change self.image each frame — scene draws it automatically.
        scale = 1
        original_image = self.images[0]
        for d_scale in [0.01, -0.01] * 10:
            if pygamejr.is_quit():
                break
            for dt in pygamejr.every_frame(30):
                scale += d_scale
                self.image = self._scale_surface(original_image, scale)
        self.image = original_image


player = Player()


def set_map(map):
    '''
    Sets the map.
    :param str map: map path, e.g. codomir.maps.linear.map1
    '''
    global tilemap, quest_scene, win_position, CHARACTER_TILES

    tilemap = pygamejr.TileMap(map)
    CHARACTER_TILES = {
        'right':  json.loads(tilemap.tmxdata.properties['character_right'])  if 'character_right'  in tilemap.tmxdata.properties else None,
        'bottom': json.loads(tilemap.tmxdata.properties['character_bottom']) if 'character_bottom' in tilemap.tmxdata.properties else None,
        'left':   json.loads(tilemap.tmxdata.properties['character_left'])   if 'character_left'   in tilemap.tmxdata.properties else None,
        'top':    json.loads(tilemap.tmxdata.properties['character_top'])    if 'character_top'    in tilemap.tmxdata.properties else None,
    }

    old_scene = quest_scene
    quest_scene = QuestScene(tilemap)

    # Move player from old scene to new scene.
    old_scene.remove(player)
    quest_scene.add(player)
    pygamejr.set_scene(quest_scene)

    player.set_position_by_tile(*_get_spawn_position())
    player._update_image()
    win_position = _get_win_position()


def init(src):
    set_map(src)


__all__ = [
    player,
    init,
    set_map,
]



import pygamejr

STATE_GAME = 'game'
STATE_GAME_OVER = 'game_over'

text = pygamejr.SubtitlesSprite(['3', '2', '1', "Hello World!"], 48)
text.rect.center = (400, 300)
text.is_visible = True

game_over_text = pygamejr.TextSprite('Game Over!', 48)
game_over_text.rect.center = (400, 300)
game_over_text.is_visible = False

state = STATE_GAME

for dt in pygamejr.every_frame():
    if state == STATE_GAME:
        text.update()
        text.rect.center = (400, 300)
        if not text.is_visible:
            state = 'game_over'
            game_over_text.is_visible = True
    elif state == STATE_GAME_OVER:
        game_over_text.rect.centerx += 1



from .resolve_path import resolve_path

tank_blue = resolve_path('images/topdown_tanks/tank_blue.png')
tank_dark = resolve_path('images/topdown_tanks/tank_dark.png')
tank_green = resolve_path('images/topdown_tanks/tank_green.png')
tank_red = resolve_path('images/topdown_tanks/tank_red.png')

tree_green = resolve_path('images/topdown_tanks/treeGreen_large.png')
tree_green_small = resolve_path('images/topdown_tanks/treeGreen_small.png')
tree_brown = resolve_path('images/topdown_tanks/treeBrown_large.png')
tree_brown_small = resolve_path('images/topdown_tanks/treeBrown_small.png')



import tkinter as tk

def get_curr_screen_geometry():
    """
    Workaround to get the size of the current screen in a multi-screen setup.

    Returns:
        geometry (str): The standard Tk geometry string.
            [width]x[height]+[left]+[top]
    """
    root = tk.Tk()
    root.update_idletasks()
    root.attributes('-fullscreen', True)
    root.state('iconic')
    geometry = root.winfo_geometry()
    scaling_factor = root.tk.call('tk', 'scaling')
    root.destroy()
    return [ int(value) for value in geometry.split('+')[0].split('x')]



import pygame
from .base import BaseSprite

# GLOBAL VARIABLES
COLOR = (255, 100, 98)
SURFACE_COLOR = (167, 255, 100)
WIDTH = 500
HEIGHT = 500


class CircleSprite(BaseSprite):
    def __init__(
            self,
            color="red",
            radius=50,
            sprite_angle: float = 0,
            scene=None,
            *args):
        super().__init__(sprite_angle, scene=scene, *args)

        self.image = pygame.Surface([radius*2, radius*2])
        self.image.set_colorkey(COLOR)
        pygame.draw.circle(self.image,
                           color,
                           pygame.Vector2(radius, radius),
                           radius
        )

        self.rect = self.image.get_frect()



from codomir import player, wait_quit, set_map, maps

set_map(maps.loop.map2)

for i in range(4):
    player.move_forward()
    player.turn_right()
    player.move_forward()
    player.turn_left()

wait_quit()



import pygame
from .base import BaseSprite

# GLOBAL VARIABLES
COLOR = (255, 100, 98)


class RectSprite(BaseSprite):
    def __init__(
            self,
            color="red",
            width=50,
            height=50,
            x=0,
            y=0,
            sprite_angle: float = 0,
            scene=None,
            *args
    ):
        super().__init__(sprite_angle, scene=scene, *args)

        self.image = pygame.Surface([width, height])
        self.image.set_colorkey(COLOR)
        pygame.draw.rect(self.image,
                           color,
                           pygame.FRect((x, y, width, height)),
        )

        self.rect = self.image.get_frect()



from codomir import player, wait_quit, set_map, maps

set_map(maps.nested_loops.map2)

for i in range(2):
    for j in range(4):
        player.move_forward()
    player.turn_right()
    player.move_forward()
    player.move_forward()
    player.turn_right()

    for j in range(4):
        player.move_forward()
    player.turn_left()
    player.move_forward()
    player.move_forward()
    player.turn_left()

wait_quit()



from .resolve_path import resolve_path

bee = resolve_path('images/enemies/bee.png')
fish_green = resolve_path('images/enemies/fishGreen.png')
fish_pink = resolve_path('images/enemies/fishPink.png')
fly = resolve_path('images/enemies/fly.png')
frog = resolve_path('images/enemies/frog.png')
frog_move = resolve_path('images/enemies/frog_move.png')
ladybug = resolve_path('images/enemies/ladybug.png')
mouse = resolve_path('images/enemies/mouse.png')
saw = resolve_path('images/enemies/saw.png')
saw_half = resolve_path('images/enemies/sawHalf.png')
slime_block = resolve_path('images/enemies/slimeBlock.png')
slime_blue = resolve_path('images/enemies/slimeBlue.png')
slime_blue_move = resolve_path('images/enemies/slimeBlue_move.png')
slime_green = resolve_path('images/enemies/slimeGreen.png')
slime_purple = resolve_path('images/enemies/slimePurple.png')
worm_green = resolve_path('images/enemies/wormGreen.png')
worm_green_dead = resolve_path('images/enemies/wormGreen_dead.png')
worm_green_move = resolve_path('images/enemies/wormGreen_move.png')
worm_pink = resolve_path('images/enemies/wormPink.png')
coin_bronze = resolve_path('images/items/coinBronze.png')
coin_gold = resolve_path('images/items/gold_1.png')
coin_gold_ll = resolve_path('images/items/coinGold_ll.png')
coin_gold_lr = resolve_path('images/items/coinGold_lr.png')
coin_gold_ul = resolve_path('images/items/coinGold_ul.png')
coin_gold_ur = resolve_path('images/items/coinGold_ur.png')
coin_silver = resolve_path('images/items/coinSilver_test.png')
coin_silver_test = resolve_path('images/items/coinSilver_test.png')
flag_green1 = resolve_path('images/items/flagGreen1.png')
flag_green2 = resolve_path('images/items/flagGreen2.png')
flag_green_down = resolve_path('images/items/flagGreen_down.png')
flag_red1 = resolve_path('images/items/flagRed1.png')
flag_red2 = resolve_path('images/items/flagRed2.png')
flag_red_down = resolve_path('images/items/flagRed_down.png')
flag_yellow1 = resolve_path('images/items/flagYellow1.png')
flag_yellow2 = resolve_path('images/items/flagYellow2.png')
flag_yellow_down = resolve_path('images/items/flagYellow_down.png')
gem_blue = resolve_path('images/items/gemBlue.png')
gem_green = resolve_path('images/items/gemGreen.png')
gem_red = resolve_path('images/items/gemRed.png')
gem_yellow = resolve_path('images/items/gemYellow.png')
gold_1 = resolve_path('images/items/gold_1.png')
gold_2 = resolve_path('images/items/gold_2.png')
gold_3 = resolve_path('images/items/gold_3.png')
gold_4 = resolve_path('images/items/gold_4.png')
key_blue = resolve_path('images/items/keyBlue.png')
key_green = resolve_path('images/items/keyGreen.png')
key_red = resolve_path('images/items/keyRed.png')
key_yellow = resolve_path('images/items/keyYellow.png')
ladder_mid = resolve_path('images/items/ladderMid.png')
ladder_top = resolve_path('images/items/ladderTop.png')
star = resolve_path('images/items/star.png')
laser_blue01 = resolve_path('images/space_shooter/laserBlue01.png')
laser_red01 = resolve_path('images/space_shooter/laserRed01.png')
meteor_grey_big1 = resolve_path('images/space_shooter/meteorGrey_big1.png')
meteor_grey_big2 = resolve_path('images/space_shooter/meteorGrey_big2.png')
meteor_grey_big3 = resolve_path('images/space_shooter/meteorGrey_big3.png')
meteor_grey_big4 = resolve_path('images/space_shooter/meteorGrey_big4.png')
meteor_grey_med1 = resolve_path('images/space_shooter/meteorGrey_med1.png')
meteor_grey_med2 = resolve_path('images/space_shooter/meteorGrey_med2.png')
meteor_grey_small1 = resolve_path('images/space_shooter/meteorGrey_small1.png')
meteor_grey_small2 = resolve_path('images/space_shooter/meteorGrey_small2.png')
meteor_grey_tiny1 = resolve_path('images/space_shooter/meteorGrey_tiny1.png')
meteor_grey_tiny2 = resolve_path('images/space_shooter/meteorGrey_tiny2.png')
player_life1_orange = resolve_path('images/space_shooter/playerLife1_orange.png')
player_ship1_green = resolve_path('images/space_shooter/playerShip1_green.png')
player_ship1_orange = resolve_path('images/space_shooter/playerShip1_orange.png')
player_ship2_orange = resolve_path('images/space_shooter/playerShip2_orange.png')
player_ship3_orange = resolve_path('images/space_shooter/playerShip3_orange.png')




from codomir import player, wait_quit, set_map, maps

set_map(maps.loop.map3)

for i in range(2):
    player.turn_left()

for i in range(2):
    player.move_forward()
    player.move_forward()
    player.turn_left()
    player.move_forward()
    player.turn_right()

wait_quit()



import pygame
import pygamejr

bee = pygamejr.ImageSprite(pygamejr.resources.image.bee)

SPEED = 5

for frame in pygamejr.every_frame():

    # Получаем состояние всех клавиш
    keys = pygame.key.get_pressed()

    # Двигаем пчелу в зависимости от нажатых клавиш
    if keys[pygame.K_w]:
        new_top = bee.rect.top - SPEED
        if new_top > 0:
            bee.rect.top = new_top
    if keys[pygame.K_s]:
        new_bottom = bee.rect.bottom + SPEED
        if new_bottom < pygamejr.screen.get_height():
            bee.rect.bottom = new_bottom
    if keys[pygame.K_a]:
        new_left = bee.rect.left - SPEED
        if new_left > 0:
            bee.rect.left = new_left
    if keys[pygame.K_d]:
        new_right = bee.rect.right + SPEED
        if new_right < pygamejr.screen.get_width():
            bee.rect.right = new_right



import random
import pygamejr
import bullets
import enemy_bullets

# враги
enemies = []
for top in range(50, 200, 50):
   for left in range(100, pygamejr.screen.get_width() - 100, 50):
       enemy = pygamejr.ImageSprite("../assets/InvaderA_00.png")
       enemy.rect.top = top
       enemy.rect.left = left
       enemies.append(enemy)

vertical_speed = 0.1

def move_vertically():
    '''Движение врагов по вертикали'''
    for enemy in enemies:
        enemy.rect.centery += vertical_speed
        if enemy.rect.centery >= pygamejr.screen.get_height()-200:
            print('game over')

horizontal_speed = 0.5
horizontal_limit = 0

def move_horizontally():
   '''Движение врагов по горизонтали'''
   global horizontal_speed
   global horizontal_limit
   for enemy in enemies:
       enemy.rect.centerx += horizontal_speed

   horizontal_limit += horizontal_speed
   if horizontal_limit > 80 or horizontal_limit == 0:
       horizontal_speed *= -1

def is_hit_enemy_by_bullet():
   '''проверяет попали ли пуля во врага'''
   for enemy in enemies:
       if bullets.is_hit_enemy(enemy):
           enemy.is_visible = False
           enemies.remove(enemy)
           return True
   return False

def fire():
    if enemies:
        enemy = random.choice(enemies)
        enemy_bullets.fire(enemy)



from codomir import player, wait_quit, set_map, maps

set_map(maps.linear.map5)

player.move_forward()
player.move_forward()

player.turn_left()
player.move_forward()
player.move_forward()
player.turn_left()
player.move_forward()

wait_quit()



import pygamejr
import pygame
import bullets
import enemies
import fortifications
import enemy_bullets

# корабль
ship = pygamejr.ImageSprite('../assets/Ship.png')
ship.rect.bottom = pygamejr.screen.get_height() - 50


for frame in pygamejr.every_frame():
    keys = pygame.key.get_pressed()

    if keys[pygame.K_SPACE]:
        bullets.fire(ship)

    if keys[pygame.K_d] and ship.rect.right <= pygamejr.screen.get_width():
        ship.rect.centerx += 3

    if keys[pygame.K_a] and ship.rect.left >= 0:
        ship.rect.centerx -= 3

    enemies.is_hit_enemy_by_bullet()
    enemies.move_vertically()
    enemies.move_horizontally()

    bullets.fly()

    fortifications.is_hit_by_bullet()
    fortifications.is_hit_by_enemy_bullet()

    enemies.fire()
    enemy_bullets.fly()




from ..resources.resolve_path import resolve_path

map1 =  resolve_path('kenney_sokoban/map_loop1.tmx')
map2 = resolve_path('kenney_sokoban/map_loop2.tmx')
map3 = resolve_path('kenney_sokoban/map_loop3.tmx')
map4 = resolve_path('kenney_sokoban/map_loop4.tmx')
map5 = resolve_path('kenney_sokoban/map_loop5.tmx')
map6 = resolve_path('kenney_sokoban/map_loop6.tmx')


import pygame
import pygamejr

# =================================================================
# GLOBAL SCENE
#
# Sprites created before the first set_scene call are placed
# in the global scene and drawn ON TOP of any current scene.
# Player and HUD live here — they are visible across all levels.
# =================================================================

player = pygamejr.ImageSprite(pygamejr.resources.image.player_ship1_orange)
player.rect.center = (400, 500)

hud_score = pygamejr.TextSprite("Score: 0", size=24)
hud_score.rect.topleft = (10, 10)

hud_level = pygamejr.TextSprite("Level 1", size=24)
hud_level.rect.topright = (790, 10)

score = 0

# =================================================================
# MENU SCENE
# =================================================================
menu_scene = pygamejr.Scene()
pygamejr.set_scene(menu_scene)

pygamejr.TextSprite("Space Shooter", size=56).rect.center = (400, 200)
pygamejr.TextSprite("SPACE - start game", size=30).rect.center = (400, 320)

# =================================================================
# LEVEL 1 — bees and ladybugs
# =================================================================
level1_scene = pygamejr.Scene()
pygamejr.set_scene(level1_scene)

bee1 = pygamejr.ImageSprite(pygamejr.resources.image.bee)
bee1.rect.center = (150, 150)

bee2 = pygamejr.ImageSprite(pygamejr.resources.image.bee)
bee2.rect.center = (500, 100)

pygamejr.ImageSprite(pygamejr.resources.image.ladybug).rect.center = (650, 200)

pygamejr.TextSprite(
    "ARROWS - move   |   N - next level   |   ESC - menu",
    size=20
).rect.midbottom = (400, 590)

# =================================================================
# LEVEL 2 — flies and frogs
# =================================================================
level2_scene = pygamejr.Scene()
pygamejr.set_scene(level2_scene)

fly1 = pygamejr.ImageSprite(pygamejr.resources.image.fly)
fly1.rect.center = (180, 130)

fly2 = pygamejr.ImageSprite(pygamejr.resources.image.fly)
fly2.rect.center = (520, 160)

pygamejr.ImageSprite(pygamejr.resources.image.frog).rect.center = (340, 80)
pygamejr.ImageSprite(pygamejr.resources.image.worm_green).rect.center = (650, 200)

pygamejr.TextSprite(
    "ARROWS - move   |   P - previous level   |   ESC - menu",
    size=20
).rect.midbottom = (400, 590)

# =================================================================
# Start from the menu.
# Player and HUD are hidden — not needed on the menu screen.
# =================================================================
pygamejr.set_scene(menu_scene)
player.is_visible = False
hud_score.is_visible = False
hud_level.is_visible = False

SPEED = 250
prev_space = prev_n = prev_p = prev_esc = False

for dt in pygamejr.every_frame():
    keys = pygame.key.get_pressed()
    current = pygamejr.get_current_scene()

    # --- Menu ---
    if current is menu_scene:
        if keys[pygame.K_SPACE] and not prev_space:
            player.rect.center = (400, 500)
            player.is_visible = True
            hud_score.is_visible = True
            hud_level.is_visible = True
            hud_level.text = "Level 1"
            pygamejr.set_scene(level1_scene)

    # --- Game levels ---
    else:
        # Player movement works on any level because
        # player lives in the global scene.
        if keys[pygame.K_LEFT]:
            player.rect.x -= SPEED * dt
        if keys[pygame.K_RIGHT]:
            player.rect.x += SPEED * dt
        if keys[pygame.K_UP]:
            player.rect.y -= SPEED * dt
        if keys[pygame.K_DOWN]:
            player.rect.y += SPEED * dt
        player.rect.clamp_ip(pygamejr.screen.get_rect())

        # Animate level 1 enemies
        if current is level1_scene:
            bee1.rect.x += 120 * dt
            bee2.rect.x -= 90 * dt
            if bee1.rect.left > pygamejr.screen.get_width():
                bee1.rect.right = 0
            if bee2.rect.right < 0:
                bee2.rect.left = pygamejr.screen.get_width()

        # Animate level 2 enemies
        if current is level2_scene:
            fly1.rect.x += 140 * dt
            fly2.rect.x -= 100 * dt
            if fly1.rect.left > pygamejr.screen.get_width():
                fly1.rect.right = 0
            if fly2.rect.right < 0:
                fly2.rect.left = pygamejr.screen.get_width()

        # Level 1 -> Level 2
        # Player position is preserved automatically on transition!
        if current is level1_scene and keys[pygame.K_n] and not prev_n:
            score += 100
            hud_score.text = f"Score: {score}"
            hud_level.text = "Level 2"
            pygamejr.set_scene(level2_scene)

        # Level 2 -> Level 1
        if current is level2_scene and keys[pygame.K_p] and not prev_p:
            score += 100
            hud_score.text = f"Score: {score}"
            hud_level.text = "Level 1"
            pygamejr.set_scene(level1_scene)

        # Return to menu
        if keys[pygame.K_ESCAPE] and not prev_esc:
            player.is_visible = False
            hud_score.is_visible = False
            hud_level.is_visible = False
            pygamejr.set_scene(menu_scene)

    prev_space = keys[pygame.K_SPACE]
    prev_n = keys[pygame.K_n]
    prev_p = keys[pygame.K_p]
    prev_esc = keys[pygame.K_ESCAPE]



import pathlib
import pygame
from .base import BaseSprite


def _crop_alpha(surface):
    """
    Обрезает прозрачные области вокруг изображения.
    Возвращает новый Surface, содержащий только видимую часть.
    """
    rect = surface.get_bounding_rect()
    cropped = surface.subsurface(rect)
    # Делаем копию, чтобы получить независимый Surface
    return cropped.copy()


class ImageSprite(BaseSprite):
    def __init__(
            self,
            filename: str | pathlib.Path = None,
            image: pygame.Surface = None,
            crop_alpha: bool = True,
            sprite_angle: float = 0,
            scene=None,
            *args):
        super().__init__(sprite_angle, scene=scene, *args)
        if filename:
            self._original_image = pygame.image.load(filename).convert_alpha()
        else:
            self._original_image = image
        if crop_alpha:
            self._original_image = _crop_alpha(self._original_image)
        self.image = self._original_image
        self.rect = self.image.get_frect()
        self.mask = pygame.mask.from_surface(self.image)

    def rotate(self, angle: float):
        super().rotate(angle)
        self.image = pygame.transform.rotate(self._original_image, self._angle)
        self.rect = self.image.get_frect(center=self.rect.center)
        self.mask = pygame.mask.from_surface(self.image)

    def turn_left(self, angle: float = 1):
        self.rotate(-angle)

    def turn_right(self, angle: float = 1):
        self.rotate(angle)



import pygame
from .base import BaseSprite

# GLOBAL VARIABLES
COLOR = (255, 100, 98)
SURFACE_COLOR = (167, 255, 100)
WIDTH = 500
HEIGHT = 500


class TextSprite(BaseSprite):
    def __init__(
            self,
            text: str='',
            size: int = 32,
            color=(255, 255, 255),
            font_name=None,
            sprite_angle: float = 0,
            scene=None,
            *args):
        super().__init__(sprite_angle, scene=scene, *args)
        self._text = text
        self._size = size
        self._color = color
        self._font = pygame.font.Font(font_name, size)
        self._render_text()

    def _render_text(self):
        try:
            old_topleft = self.rect.topleft
        except AttributeError:
            old_topleft = None
        self._original_image = self._font.render(self._text, True, self._color)
        self.image = self._original_image
        self.rect = self.image.get_frect()
        if old_topleft is not None:
            self.rect.topleft = old_topleft
        self.mask = pygame.mask.from_surface(self.image)

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, value):
        if self._text != value:
            self._text = value
            self._render_text()



import time
import pygamejr

# снаряды
bullets = []

#время перезарядки
RELOAD_TIME = 0.5

#время последнего выстрела
last_fire_time = 0

def fire(gun_sprite):
    """
    Выстрел пулей
    :param gun_sprite: спрайт, который стреляет
    """
    global last_fire_time  # 4
    time_now = time.time()

    if time_now - last_fire_time >= RELOAD_TIME:
        is_reloading = False
    else:
        is_reloading = True

    # "выстрел" - добавление нового снаряда в общий список
    if not is_reloading:
        bullet = pygamejr.CircleSprite(color="white", radius=3)
        bullet.rect.centerx = gun_sprite.rect.centerx
        bullet.rect.bottom = gun_sprite.rect.top
        bullets.append(bullet)
        last_fire_time = time_now

BULLET_SPEED = 3
def fly():
    """
    Полёт пули
    """
    bullet_for_delete = None

    for bullet in bullets:
        bullet.rect.y += BULLET_SPEED
        if bullet.rect.y >= pygamejr.screen.get_height():
            bullet_for_delete = bullet

    if bullet_for_delete:
        bullet_for_delete.kill()
        bullet_for_delete.is_visible = False
        bullets.remove(bullet_for_delete)

def is_hit_enemy(enemy_sprite):  # 5
   """
   Простая функция проверки, попала ли пуля во врага.
   :param enemy_sprite: спрайт врага
   :return: True если одна из пуль попала во врага, False если ни одна пуля не попала.
   """

   for bullet in bullets:
       if bullet.rect.colliderect(enemy_sprite):
           bullet.is_visible = False
           bullets.remove(bullet)
           return True
   return False


import pygamejr

ball = pygamejr.CircleSprite()
bee = pygamejr.ImageSprite(pygamejr.resources.image.bee)
bee.rect.centerx = pygamejr.screen.get_width() / 2
bee.rect.centery = pygamejr.screen.get_height() / 2

while pygamejr.next_frame():
    ball.rect.x += 0.5
    bee.rect.x += 0.5



import time
import pygame
from .text import TextSprite

# GLOBAL VARIABLES
COLOR = (255, 100, 98)
SURFACE_COLOR = (167, 255, 100)
WIDTH = 500
HEIGHT = 500


class SubtitlesSprite(TextSprite):
    def __init__(
            self,
            text_list: list[str] = [],
            size: int = 32,
            color=(255, 255, 255),
            font_name=None,
            sprite_angle: float = 0,
            scene=None,
            *args):
        super().__init__(text_list[0], size, color, font_name, sprite_angle, scene, *args)
        self._text_list = text_list
        self._text_index = 0
        self._last_update = time.time()
        self.visible = 0

    @property
    def visible(self):
        return super().visible

    @visible.setter
    def visible(self, value):
        old = super().visible
        pygame.sprite.DirtySprite.visible.fset(self, value)
        if value and not old:
            self._last_update = time.time()
            self._render_text()

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, value):
        if self._text != value:
            self._text = value
            self._render_text()

    def update(self):
        if not self.visible:
            return

        current_time = time.time()
        if current_time - self._last_update > 3:
            print(self._text_index, len(self._text_list))
            if self._text_index >= len(self._text_list) - 1:
                self._text_index = 0
                self.visible = 0
            else:
                self._text_index += 1
                self.text = self._text_list[self._text_index]
                self._render_text()
            self._last_update = current_time



from codomir import player, wait_quit, set_map, maps

set_map(maps.loop.map5)

player.turn_left()

for i in range(5):
    player.move_forward()
    player.turn_left()
    player.move_forward()
    player.turn_right()

wait_quit()



import pygame
import pygamejr

# =================================================================
# GLOBAL SCENE
# Sprites created here are visible in every scene.
# =================================================================

score = 0

player = pygamejr.ImageSprite(pygamejr.resources.image.player_ship1_orange)
player.rect.center = (400, 300)

hud_score = pygamejr.TextSprite("Score: 0", size=24)
hud_score.rect.topleft = (10, 10)

hud_level = pygamejr.TextSprite("Level 1", size=24)
hud_level.rect.topright = (790, 10)

hud_status = pygamejr.TextSprite("", size=48)
hud_status.rect.center = (400, 420)
hud_status.is_visible = False

# =================================================================
# SCENE CLASSES
# =================================================================

class MenuScene(pygamejr.Scene):
    def __init__(self):
        super().__init__()
        self._prev_space = False

        title1 = pygamejr.TextSprite("Space Shooter", size=56, scene=self)
        title1.rect.center = (400, 200)
        title2 = pygamejr.TextSprite("SPACE - start game", size=30, scene=self)
        title2.rect.center = (400, 320)

    def update(self, dt=0):
        super().update(dt)



class LevelScene(pygamejr.Scene):
    """Base class for game levels. Handles movement, coin collection and enemy collision."""
    SPEED = 250
    _COIN_POSITIONS = []

    def __init__(self, player):
        super().__init__()
        self.player = player
        self.state = None
        self._prev_esc = False
        self.enemies = pygame.sprite.Group()
        self.coins = pygame.sprite.Group()

        # Coins (static)
        self._create_coins()

    def _create_coins(self):
        for pos in self._COIN_POSITIONS:
            coin = pygamejr.ImageSprite(pygamejr.resources.image.coin_gold, scene=self)
            coin.rect.center = pos
            self.coins.add(coin)

    def init_scene(self):
        self.state = None
        for coin in list(self.coins):
            coin.kill()
        self.coins.empty()
        self._create_coins()
        return super().init_scene()

    def update(self, dt=0):
        global score
        super().update(dt)
        keys = pygame.key.get_pressed()
        p = self.player
        if keys[pygame.K_LEFT]:
            p.rect.x -= self.SPEED * dt
        if keys[pygame.K_RIGHT]:
            p.rect.x += self.SPEED * dt
        if keys[pygame.K_UP]:
            p.rect.y -= self.SPEED * dt
        if keys[pygame.K_DOWN]:
            p.rect.y += self.SPEED * dt
        p.rect.clamp_ip(pygamejr.screen.get_rect())

        if self.state or not self.player.is_visible:
            return

        # Coin collection
        collected = pygame.sprite.spritecollide(
            self.player, self.coins, True, pygame.sprite.collide_mask
        )
        if collected:
            score += 10 * len(collected)
            if not self.coins:
                self.state = "win"
                return

        # Enemy collision → game over
        if pygame.sprite.spritecollide(
            self.player, self.enemies, False, pygame.sprite.collide_mask
        ):
            self.state = "game_over"


class Level1Scene(LevelScene):
    _COIN_POSITIONS = [(200, 250), (560, 180), (400, 430)]

    def __init__(self, player):
        super().__init__(player)

        # Enemies (moving)
        self.bee1 = pygamejr.ImageSprite(pygamejr.resources.image.bee, scene=self)
        self.bee1.rect.center = (150, 150)
        self.bee2 = pygamejr.ImageSprite(pygamejr.resources.image.bee, scene=self)
        self.bee2.rect.center = (600, 220)
        self.enemies.add(self.bee1, self.bee2)

        title = pygamejr.TextSprite(
            "ARROWS - move   |   собери монетки!   |   ESC - меню",
            size=20,
            scene=self
        )
        title.rect.midbottom = (400, 590)

    def update(self, dt=0):
        super().update(dt)
        # Animate enemies
        self.bee1.rect.x += 120 * dt
        self.bee2.rect.x -= 90 * dt
        if self.bee1.rect.left > pygamejr.screen.get_width():
            self.bee1.rect.right = 0
        if self.bee2.rect.right < 0:
            self.bee2.rect.left = pygamejr.screen.get_width()

    def init_scene(self):
        self.bee1.rect.center = (150, 150)
        self.bee2.rect.center = (600, 220)
        return super().init_scene()


class Level2Scene(LevelScene):
    _COIN_POSITIONS = [(150, 350), (400, 200), (640, 420)]

    def __init__(self, player):
        super().__init__(player)

        # Enemies (moving)
        self.fly1 = pygamejr.ImageSprite(pygamejr.resources.image.fly, scene=self)
        self.fly1.rect.center = (180, 130)
        self.fly1_vel = [140, 110]
        self.fly2 = pygamejr.ImageSprite(pygamejr.resources.image.fly, scene=self)
        self.fly2.rect.center = (520, 340)
        self.fly2_vel = [-100, -80]
        self.enemies.add(self.fly1, self.fly2)

        title = pygamejr.TextSprite(
            "ARROWS - move   |   собери монетки!   |   ESC - меню",
            size=20,
            scene=self,
        )
        title.rect.midbottom = (400, 590)

    def update(self, dt=0):
        super().update(dt)
        screen_rect = pygamejr.screen.get_rect()
        # Animate enemies with bounce
        for fly, vel in [(self.fly1, self.fly1_vel), (self.fly2, self.fly2_vel)]:
            fly.rect.x += vel[0] * dt
            fly.rect.y += vel[1] * dt
            if fly.rect.left < screen_rect.left or fly.rect.right > screen_rect.right:
                vel[0] *= -1
                fly.rect.clamp_ip(screen_rect)
            if fly.rect.top < screen_rect.top or fly.rect.bottom > screen_rect.bottom:
                vel[1] *= -1
                fly.rect.clamp_ip(screen_rect)

    def init_scene(self):
        self.fly1.rect.center = (180, 130)
        self.fly1_vel = [140, 110]
        self.fly2.rect.center = (520, 340)
        self.fly2_vel = [-100, -80]
        return super().init_scene()

# =================================================================
# CREATE SCENES
# =================================================================

menu_scene   = MenuScene()
level1_scene = Level1Scene(player)
level2_scene = Level2Scene(player)

# =================================================================
# START — player and HUD hidden until game begins
# =================================================================

pygamejr.set_scene(menu_scene)
player.is_visible    = False
hud_score.is_visible = False
hud_level.is_visible = False

for dt in pygamejr.every_frame():

    hud_score.text = f"Score: {score}"
    keys = pygame.key.get_pressed()

    current_scene = pygamejr.get_current_scene()

    if current_scene == menu_scene:
        if keys[pygame.K_SPACE]:
            pygamejr.set_scene(level1_scene)
            player.is_visible = True
            hud_score.is_visible = True
            hud_level.is_visible = True
            hud_status.is_visible = False
    elif current_scene in [level1_scene, level2_scene]:
        if keys[pygame.K_ESCAPE]:
            pygamejr.set_scene()
        if current_scene.state == 'win':
            if current_scene == level1_scene:
                pygamejr.set_scene(level2_scene)
                hud_level.text = 'Level 2'
            elif current_scene == level2_scene:
                player.is_visible = False
                hud_score.is_visible = False
                hud_level.is_visible = False
                hud_status.text = f"YOU WIN! {score} pts"
                hud_status.is_visible = True
                pygamejr.set_scene(None)
                pygamejr.set_scene(menu_scene)
        elif current_scene.state == 'game_over':
            hud_status.text = "GAME OVER"
            hud_status.is_visible = True
            player.is_visible = False
            pygamejr.set_scene(menu_scene)



import time
import pygamejr
import pygame

tank = pygamejr.ImageSprite(pygamejr.resources.image_tanks.tank_green)
tank.rect.centerx = pygamejr.screen.get_width() / 2
tank.rect.centery = pygamejr.screen.get_height() / 2

tree = pygamejr.ImageSprite(pygamejr.resources.image_tanks.tree_green)


for frame in pygamejr.every_frame():

    keys = pygame.key.get_pressed()


    if keys[pygame.K_a]:
        tank.turn_right()
    if keys[pygame.K_d]:
        tank.turn_left()

    if pygame.sprite.collide_mask(tank, tree):
        if keys[pygame.K_a]:
            tank.turn_left()
        if keys[pygame.K_d]:
            tank.turn_right()

    rect = tank.rect.copy()
    if keys[pygame.K_w]:
        tank.move_forward()
    if keys[pygame.K_s]:
        tank.move_forward(-1)

    if pygame.sprite.collide_mask(tank, tree):
        tank.rect = rect






from math import sin, cos, radians
import pygame
from ..base import screen, get_current_scene, get_global_scene

# GLOBAL VARIABLES
COLOR = (255, 100, 98)
SURFACE_COLOR = (167, 255, 100)
WIDTH = 500
HEIGHT = 500


class BaseSprite(pygame.sprite.DirtySprite):
    def __init__(self, sprite_angle: float = 0, is_visible: bool = True, scene=None, *args):
        super().__init__(*args)
        super()._set_visible(1 if is_visible else 0)
        self._sprite_angle = sprite_angle
        self._angle = sprite_angle
        (scene or get_current_scene() or get_global_scene()).add(self)

    def __del__(self):
        self.kill()

    @property
    def is_visible(self):
        return bool(self.visible)

    @is_visible.setter
    def is_visible(self, value):
        self.visible = 1 if value else 0

    def draw(self, draw_rect: bool = False):
        if self.is_visible:
            screen.blit(self.image, self.rect)
            if draw_rect:
                pygame.draw.rect(screen, (255, 0, 0), self.rect, 1)

    def move_forward(self, distance: float = 1.0):
        self.rect.x += distance * sin(radians(self._angle))
        self.rect.y += distance * cos(radians(self._angle))

    def rotate(self, angle: float):
        self._angle += angle
        self._angle %= 360
